# add modules folder to interpreter path
import sys
import os
modules_path = '../../modules/'
if os.path.exists(modules_path):
    sys.path.insert(1, modules_path)

# init logging to file
import logging
LOG_FORMAT = ('%(levelname) -5s %(asctime)s %(name)s:%(funcName) -35s %(lineno) -5d:  %(message)s')
logging.basicConfig(
    filename='error.log', 
    level=logging.INFO, format=LOG_FORMAT)
LOGGER = logging.getLogger(__name__)

import connexion
from multiprocessing import Process
from messaging.MessageReceiver import MessageReceiver
from messaging.MessageSender import MessageSender
from MessageList import MessageList

messages = MessageList.getInstance()

message_sender = MessageSender()
message_sender.connect()
message_sender.create_exchange('inhub', 'direct')

# init message handler
def message_received_callback(channel, method, properties, body):
    messages.appendMessage(body)
    print(f"### Received: {body}")

def pika_error_callback(error):
    # TODO gracefully handle error
    print("Rabbit MQ error!")
    os._exit(1)

# load swagger config 
app = connexion.App(__name__, specification_dir='configs/')
app.add_api('swagger.yml')

@app.route('/', methods=['GET'])
def api_root():
    return 'Endpoint of SMART RESTful API Gateway!'

# start app
if __name__ == '__main__':
    message_rec = MessageReceiver(exchange_name='rest-gateway', exchange_type='direct', queue_name='rest-gateway', auto_ack=True)
    message_rec.start(message_received_callback, pika_error_callback)

    app.run(host='0.0.0.0', port=5000, debug=True, use_reloader=False) # disable reloader so only subscribed once to rabbitmq